//
//  ExternalAuth.swift
//  HIVE_SDK_Sample
//
//  Created by sjyong on 2022/02/09.
//  Copyright © 2022 gcp. All rights reserved.
//

import Foundation
import HIVECore

public class ExternalAuth {

    typealias ExternalAuthResponseHandler = (_ response: ExternalAuthResponse?) -> Void

    let networkSerialQueue = DispatchQueue(label: "ExternalAuth.networkSerialQueue")

    static let shared: ExternalAuth = {
        let instance = ExternalAuth()

        return instance
    }()

    enum authType: String, CaseIterable {
        case signInIdp = "SIGNINIDP"
        case connect   = "CONNTECT" // 다음 피처
    }

    let signInIdpURL: [ZoneType: String] =  [
        .test: "http://test-misample.com2us.net/account/custom_auth",
        .sandbox: "http://sandbox-misample.com2us.net/account/custom_auth",
        .real: "http://misample.com2us.net/account/custom_auth"
    ]

    enum requestParamsKeys: String, CaseIterable {
        case appId = "appid"   // 앱아이디
        case idpId = "idp_id"   // CUSTOM_TWITTER, CUSTOM_NAVER
        case idpUserId = "idp_user_id"  // idp_id에서 로그인 가능한 user_id. 서버 팀에서 CUSTOM_{$PROVIDER} 특정 계정을 위해 발급한 유저 아이디
        case authType = "auth_type"
    }

    class ExternalAuthResponse {
        var resultCode = 0
        var resutMsg = ""
        var resultData: [String: Any]?
        var authKey = ""

        func isSuccess() -> Bool {
            if resultCode == 200 {
                return true
            }
            return false
        }

        var description: String {
            """
            resultCode = \(self.resultCode),
            resultMsg = \(self.resutMsg),
            resultData = \(self.resultData?.toJSON() ?? ""),
            authKey = \(self.authKey)
            """
        }
    }

    func requestSignInIdp(zoneType: ZoneType, appId: String, idpId: String, idpUserId: String, responseHandler: ExternalAuthResponseHandler?) {
        guard let url = URL(string: signInIdpURL[zoneType] ?? "") else {
            print("[ExternalAuth] requestExternalAuth invalid URL.")
            return
        }

        let parameters: [String: Any] = [
            requestParamsKeys.appId.rawValue: appId,
            requestParamsKeys.idpId.rawValue: idpId,
            requestParamsKeys.idpUserId.rawValue: idpUserId,
            requestParamsKeys.authType.rawValue: authType.signInIdp.rawValue
        ]

        self.requestAPI(url: url, parameters: parameters, responseHandler: responseHandler)
    }


    func requestAPI(url: URL, parameters: [String: Any], responseHandler: ExternalAuthResponseHandler?) {

        var request = URLRequest(url: url)
        request.httpMethod = "POST"

        let paramsData = try? JSONSerialization.data(withJSONObject: parameters, options: .fragmentsAllowed)
        request.httpBody = paramsData

        self.networkSerialQueue.async {

            let task = URLSession.shared.dataTask(with: request) { data, _, error in

                let authResponse = ExternalAuthResponse()

                if let error = error {
                    print("[ExternalAuth] requestExternalSignInIdp error occured. \(error.localizedDescription)")
                    authResponse.resultCode = error._code
                    authResponse.resutMsg = error.localizedDescription
                    responseHandler?(authResponse)
                    return
                }

                guard
                    let data = data,
                    let responseDict = try? JSONSerialization.jsonObject(with: data, options: .fragmentsAllowed) as? [String: Any]
                else {
                    print("[ExternalAuth] requestExternalSignInIdp invalid response data.")
                    responseHandler?(nil)
                    return
                }

                if let resultCode = responseDict["result_code"] as? Int {
                    authResponse.resultCode = resultCode
                }
                if let resultMsg = responseDict["result_msg"] as? String {
                    authResponse.resutMsg = resultMsg
                }
                if let resultData = responseDict["result_data"] as? [String: Any] {
                    authResponse.resultData = resultData

                    if let authKey = resultData["auth_key"] as? String {
                        authResponse.authKey = authKey
                    }
                }

                print("[ExternalAuth] \(authResponse.description)")
                responseHandler?(authResponse)
            }
            task.resume()
        }
    }
}
